@@ -1,11 +1,12 @@
 binseg <- structure(function # Binary segmentation
 ### Efficient C++ implementation of the classic binary segmentation
-### algorithm for finding changepoints in a sequence of N data. Output
-### includes columns which can be used to compute parameters for a
-### single model in log-linear time, using coef method.
+### algorithm for finding changepoints in a sequence of N data, which
+### attempt to minimize a given loss function. Output includes columns
+### which can be used to compute parameters for a single model in
+### log-linear time, using coef method.
 (distribution.str,
-### String indicating distribution, use get_distribution_info to see
-### possible values.
+### String indicating distribution/loss function, use
+### get_distribution_info to see possible values.
   data.vec,
 ### Vector of numeric data to segment.
   max.segments=NULL,
@@ -24,10 +25,10 @@ binseg <- structure(function # Binary segmentation
   min.segment.length=NULL,
 ### Positive integer, minimum number of data points per
 ### segment. Default NULL means to use min given distribution.str.
-  container.str="multiset"
+  container.str="priority_queue"
 ### C++ container to use for storing breakpoints/cost. Most users
-### should leave this at the default "multiset" for efficiency but you
-### could use "list" if you want to study the time complexity of a
+### should leave this at the default "priority_queue" for efficiency, but you
+### could use "list" if you want to study the time complexity of an asymptotically
 ### slower implementation of binary segmentation.
 ){
   ##alias<< binsegRcpp
@@ -90,8 +91,9 @@ binseg <- structure(function # Binary segmentation
     is.validation.vec, position.vec)
   na <- function(x)ifelse(x<0, NA, x)
   ##value<< list of class binsegRcpp with elements min.segment.length,
-  ##distribution.str, param.names, subtrain.borders and splits, which
-  ##is a data.table with columns:
+  ##distribution.str, param.names, subtrain.borders (positions used
+  ##for assignment of segment means to validation data) and splits,
+  ##which is a data.table with columns:
   dt <- with(result, list(
     min.segment.length=min.segment.length,
     distribution.str=distribution.str,
@@ -99,8 +101,8 @@ binseg <- structure(function # Binary segmentation
     subtrain.borders=subtrain.borders,
     splits=data.table(
       segments=1:max.segments,##<< number of segments
-      loss,##<< subtrain loss
-      validation.loss,##<< validation loss
+      loss,##<< total subtrain loss
+      validation.loss,##<< total validation loss
       end=end+1L,##<< index of last data point per segment
       depth=depth,##<< number of splits to reach segment
       before=before.param.mat,##<< params before changepoint
