@@ -1,4 +1,5 @@
 #include "binseg.h"
+#include <cmath>
 #include "PiecewiseFunction.h"
 
 Split::Split() {}
@@ -49,11 +50,11 @@ void Set::write_cumsums(int write_index){
   weighted_squares.cumsum_vec[write_index] = total_weighted_squares;
 }
 
-static dist_map_type dist_map;
-// we need get_dist_map in this file to query map contents from
+static dist_umap_type dist_umap;
+// we need get_dist_umap in this file to query map contents from
 // interface.cpp
-dist_map_type* get_dist_map(void){
-  return &dist_map;
+dist_umap_type* get_dist_umap(void){
+  return &dist_umap;
 }
 
 class CumDistribution : public Distribution {
@@ -138,8 +139,8 @@ public:
       var_param = VARIANCE;						\
       description = DESC;                                               \
       param_names_vec.push_back("mean");                                \
-      if(var_param)param_names_vec.push_back("var");                  \
-      dist_map.emplace( #NAME, this );					\
+      if(var_param)param_names_vec.push_back("var");			\
+      dist_umap.emplace( #NAME, this );					\
     }                                                                   \
   };                                                                    \
   static CONCAT(NAME,Distribution) NAME;
@@ -215,22 +216,27 @@ class absDistribution : public Distribution {
       int after_i = n_candidates-1-before_i;
       candidate_split_ptr->set_end_dist
         (first_data, first_candidate+before_i, last_data);
-      candidate_split_ptr->before.center = 
-        before_median_vec[before_i];
-      candidate_split_ptr->after.center = 
-        after_median_vec[after_i];
-      candidate_split_ptr->before.spread =
-        before_loss_vec[before_i]/before_weight_vec[before_i];
-      candidate_split_ptr->after.spread =
-        after_loss_vec[after_i]/after_weight_vec[after_i];
-      candidate_split_ptr->before.loss = adjust
-        (before_loss_vec[before_i],
-         before_weight_vec[before_i],
-         candidate_split_ptr->before.spread);
-      candidate_split_ptr->after.loss = adjust
-        (after_loss_vec[after_i],
-         after_weight_vec[after_i],
-         candidate_split_ptr->after.spread);
+      std::vector<double> *loss_ptr, *median_ptr, *weight_ptr;
+      ParamsLoss *pl_ptr;
+      int i;
+      for(int direction=0; direction<2; direction++){
+	if(direction==0){
+	  loss_ptr = &before_loss_vec;
+	  median_ptr = &before_median_vec;
+	  weight_ptr = &before_weight_vec;
+	  pl_ptr = &candidate_split_ptr->before;
+	  i = before_i;
+	}else{
+	  loss_ptr = &after_loss_vec;
+	  median_ptr = &after_median_vec;
+	  weight_ptr = &after_weight_vec;
+	  pl_ptr = &candidate_split_ptr->after;
+	  i = after_i;
+	}
+	pl_ptr->center = (*median_ptr)[i];
+	pl_ptr->spread = (*loss_ptr)[i]/(*weight_ptr)[i];
+	pl_ptr->loss = adjust((*loss_ptr)[i], (*weight_ptr)[i], pl_ptr->spread);
+      }
       best_split_ptr->maybe_update(candidate_split_ptr);
     }
   }
@@ -269,17 +275,17 @@ class absDistribution : public Distribution {
   }
 };
 
-#define ABS_DIST(NAME, DESC, VARIANCE)                          \
-  class CONCAT(NAME, Distribution) : public absDistribution {   \
-  public:                                                       \
-    CONCAT(NAME, Distribution) (){                              \
-      var_param = VARIANCE;                                   \
-      description = DESC;                                       \
-      param_names_vec.push_back("median");                      \
-      if(var_param)param_names_vec.push_back("scale");        \
-      dist_map.emplace( #NAME, this );                          \
-    }                                                           \
-  };                                                            \
+#define ABS_DIST(NAME, DESC, VARIANCE)				 \
+  class CONCAT(NAME, Distribution) : public absDistribution {	 \
+  public:							 \
+    CONCAT(NAME, Distribution) (){				 \
+      var_param = VARIANCE;					 \
+      description = DESC;					 \
+      param_names_vec.push_back("median");			 \
+      if(var_param)param_names_vec.push_back("scale");		 \
+      dist_umap.emplace( #NAME, this );                          \
+    }								 \
+  };								 \
 static CONCAT(NAME, Distribution) NAME;
 
 ABS_DIST(l1, "change in median (loss is total absolute deviation)", false)
@@ -383,40 +389,34 @@ template <typename T>
 class MyContainer : public Container {
 public:
   T segment_container;
-  typename T::iterator best;
   int get_size(void){
     return segment_container.size();
   }
-  void remove_best(void){
-    segment_container.erase(best);
-  }
-  virtual typename T::iterator get_best_it(void) = 0;  
-  const Segment* set_best(void){
-    best = get_best_it();
-    return &(*best);
-  }
 };
 
 typedef std::multiset<Segment> segment_set_type;
-static factory_map_type factory_map;
+static container_umap_type container_umap;
 ContainerFactory::ContainerFactory
 (const char *name, construct_fun_type construct, destruct_fun_type destruct){
   construct_fun_ptr = construct;
   destruct_fun_ptr = destruct;
-  factory_map.emplace(name, this);
+  container_umap.emplace(name, this);
 }
-factory_map_type* get_factory_map(void){
-  return &factory_map;
+container_umap_type* get_container_umap(void){
+  return &container_umap;
 }
 
-#define CMAKER(CONTAINER, INSERT, BEST) \
-  class CONCAT(CONTAINER,Wrapper) : public MyContainer< std::CONTAINER<Segment> > { \
+#define CMAKER(CONTAINER, STRUCT, INSERT, SET_IT, GET_SEG, ERASE)	\
+  class CONCAT(CONTAINER,Wrapper) : public MyContainer< STRUCT > {	\
   public:                                                               \
     void insert(Segment& new_seg){                                      \
       segment_container.INSERT(new_seg);                                \
     }                                                                   \
-    std::CONTAINER<Segment>::iterator get_best_it(void){                \
-      return BEST;                                                      \
+    Segment get_best(void){						\
+      SET_IT;								\
+      Segment seg = GET_SEG;						\
+      ERASE;								\
+      return seg;							\
     }                                                                   \
   };                                                                    \
   Container* CONCAT(CONTAINER,construct) (){                            \
@@ -428,9 +428,21 @@ factory_map_type* get_factory_map(void){
   static ContainerFactory CONCAT(CONTAINER,_instance)                   \
     ( #CONTAINER, CONCAT(CONTAINER,construct), CONCAT(CONTAINER,destruct) );
 
-CMAKER(multiset, insert, segment_container.begin())
+#define CIT(CONTAINER, INSERT, BEST)					\
+  CMAKER(CONTAINER, std::CONTAINER<Segment>, INSERT, std::CONTAINER<Segment>::iterator it = BEST, *it, segment_container.erase(it))
 
-CMAKER(list, push_back, std::min_element(segment_container.begin(),segment_container.end()))
+CIT(multiset, insert, segment_container.begin())
+
+CIT(list, push_back, std::min_element(segment_container.begin(),segment_container.end()))
+
+class PQ_Compare {
+public:
+  bool operator()(Segment a, Segment b){
+    return !(a < b);
+  }
+};
+#define PQ_STRUCT std::priority_queue<Segment,std::vector<Segment>,PQ_Compare>
+CMAKER(priority_queue, PQ_STRUCT, push, , segment_container.top(), segment_container.pop())
 
 class Candidates {
 public:
@@ -450,7 +462,7 @@ public:
    double *subtrain_borders, Distribution *dist_ptr,
    int min_segment_length_arg
    ){
-    factory_ptr = factory_map.at(container_str);
+    factory_ptr = container_umap.at(container_str);
     container_ptr = factory_ptr->construct_fun_ptr();
     min_segment_length = min_segment_length_arg;
     subtrain.dist_ptr = dist_ptr;
@@ -476,20 +488,26 @@ public:
 	write_subtrain = last_subtrain_i >= 0 && is_subtrain;
       }
       if(write_subtrain || write_end){
+	if(write_index==0){
+	  subtrain_borders[write_index] = position_vec[0]-0.5;
+	}
 	if(write_subtrain){
 	  pos_total = position_vec[data_i]+position_vec[last_subtrain_i];
 	  pos_change = pos_total/2;
-	  if(write_index==0){
-	    subtrain_borders[write_index] = position_vec[last_subtrain_i]-0.5;
-	  }
 	}else{
-	  pos_change = position_vec[data_i-1]+0.5;//last.
+	  pos_change = position_vec[n_data-1]+0.5;//last.
 	}
 	subtrain_borders[write_index+1] = pos_change;
 	int read_index=read_start;
 	while(read_index < n_data && position_vec[read_index] <= pos_change){
 	  double data_value = data_vec[read_index];
           double weight_value = weight_vec[read_index];
+	  if(read_index < n_data-1){
+	    double maybe_change = (position_vec[read_index]+position_vec[read_index+1])/2;
+	    if(maybe_change>pos_change){
+	      subtrain_borders[write_index+1] = maybe_change;
+	    }
+	  }
           Set *this_set;
 	  if(is_validation_vec[read_index]){
             this_set = &validation;
@@ -664,7 +682,12 @@ int binseg
       return ERROR_POSITIONS_MUST_INCREASE;
     }
   }
-  Distribution* dist_ptr = dist_map.at(distribution_str);
+  for(int data_i=0; data_i<n_data; data_i++){
+    if(!std::isfinite(data_vec[data_i])){
+      return ERROR_DATA_MUST_BE_FINITE;
+    }
+  }
+  Distribution* dist_ptr = dist_umap.at(distribution_str);
   for(int data_i=0; data_i<n_data; data_i++){
     int status = dist_ptr->check_data(data_vec[data_i]);
     if(status != 0){
@@ -723,35 +746,32 @@ int binseg
   int seg_i = 0;
   while(V.container_ptr->not_empty() && ++seg_i < max_segments){
     // Store loss and model parameters associated with this split.
-    const Segment *seg_ptr = V.container_ptr->set_best();
+    const Segment seg = V.container_ptr->get_best();
     out_arrays.save
       (seg_i, 
-       subtrain_loss[seg_i-1] + seg_ptr->best_decrease,
-       validation_loss[seg_i-1] + seg_ptr->validation_decrease,
-       seg_ptr->best_split.this_end,
-       seg_ptr->depth,
-       seg_ptr->best_split.before,
-       seg_ptr->best_split.after,
-       seg_ptr->invalidates_index,
-       seg_ptr->invalidates_after,
-       seg_ptr->best_split.this_end - seg_ptr->first_i + 1,
-       seg_ptr->last_i - seg_ptr->best_split.this_end);
+       subtrain_loss[seg_i-1] + seg.best_decrease,
+       validation_loss[seg_i-1] + seg.validation_decrease,
+       seg.best_split.this_end,
+       seg.depth,
+       seg.best_split.before,
+       seg.best_split.after,
+       seg.invalidates_index,
+       seg.invalidates_after,
+       seg.best_split.this_end - seg.first_i + 1,
+       seg.last_i - seg.best_split.this_end);
     // Finally add new split candidates if necessary.
     V.maybe_add
-      (seg_ptr->first_i, seg_ptr->best_split.this_end,
+      (seg.first_i, seg.best_split.this_end,
        0,//invalidates_after=0 => before_mean invalidated.
-       seg_i, seg_ptr->best_split.before.loss,
-       seg_ptr->before_validation_loss,
-       seg_ptr->depth);
+       seg_i, seg.best_split.before.loss,
+       seg.before_validation_loss,
+       seg.depth);
     V.maybe_add
-      (seg_ptr->best_split.this_end+1, seg_ptr->last_i,
+      (seg.best_split.this_end+1, seg.last_i,
        1,//invalidates_after=1 => after_mean invalidated.
-       seg_i, seg_ptr->best_split.after.loss,
-       seg_ptr->after_validation_loss,
-       seg_ptr->depth);
-    // Erase at end because we need seg_ptr->values during maybe_add
-    // inserts above.
-    V.container_ptr->remove_best();
+       seg_i, seg.best_split.after.loss,
+       seg.after_validation_loss,
+       seg.depth);
   }
   return 0;//SUCCESS.
 }
@@ -769,5 +789,5 @@ int get_n_subtrain
   
 param_names_type* get_param_names
 (const char *distribution_str){
-  return &(dist_map.at(distribution_str)->param_names_vec);
+  return &(dist_umap.at(distribution_str)->param_names_vec);
 }
